r"""``sphobjinv._vendored.fuzzywuzzy`` *package definition module*.

``sphobjinv`` is a toolkit for manipulation and inspection of
Sphinx |objects.inv| files.

This subpackage vendors an archival version of fuzzywuzzy from
before it incorporated python-Levenshtein and thus acquired
the GPL licence by flow-through, overriding the prior
MIT license of the package.

All code in this subpackage vendored from:
https://github.com/seatgeek/fuzzywuzzy/tree/4bf28161f7005f3aa9d4d931455ac55126918df7

Imports and code details were changed as required to make the code compatible
with current (2021) versions of Python 3.

Link to MIT-licensed fuzzywuzzy version obtained from the bottom of the README at:
https://github.com/maxbachmann/RapidFuzz/tree/460d291a38922c7fead920de8147798b817653cb


**Author**
    Brian Skinn (brian.skinn@gmail.com)

**File Created**
    11 Dec 2021

**Copyright**
    \(c) 2016-2026 Brian Skinn and community contributors

**Source Repository**
    https://github.com/bskinn/sphobjinv

**Documentation**
    https://sphobjinv.readthedocs.io/en/stable

**License**
    Code: `MIT License`_

    Docs & Docstrings: |CC BY 4.0|_

    See |license_txt|_ for full license terms.

**Members**

"""
